@@ -16,8 +16,7 @@
 #include <dbus/dbus-protocol.h>
 #include <dbus/dbus-glib.h>
 
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/errors.h>
+#include <telepathy-glib/telepathy-glib.h>
 
 #include "conn.h"
 
